#!/usr/bin/env python
#----------------------------------------------------------------------------
# Name:         wxDlgAddEditDocument.py
# Author:       Bjoern
# Created:      09/25/2001
# Copyright:    (c) 2001 SKYRiX Software AG
#----------------------------------------------------------------------------

from wxPython.wx import *
from wxSkyClient_wdr import dlgAddEditPerson, dlgAddEditEnterprise
from wxDlgGetContact import wxDlgGetContact
from wxDlgCalendarSelect import wxDlgCalendarSelect
from skyrix import Person, Enterprise
from skyrix.ui.Dialog import *
from wxSkyData import *

class wxDlgAddEditDocument(wxFrame):

  # __init__
  def __init__(self, parent, id, title, app, record = None, type = 'person',
               pos = wxPyDefaultPosition, size = wxPyDefaultSize,
               style = wxRESIZE_BORDER|wxCAPTION|wxSYSTEM_MENU):

    wxFrame.__init__(self, parent, id, title, pos, size, style)

    self.parent = app.frame
    self.record = record
    self.app = app
    self.type = type
    
    wxInitAllImageHandlers()

    if type == 'person': dlgAddEditPerson( self, true )
    else: dlgAddEditEnterprise( self, true )
    self.CentreOnParent()
  
    self.gender = getAddGenderEntries()
    self.phone  = getAddPhoneFields()
   
    # address IDs and keys
    # IDs have fixed numbers, beginning with 20000
    # private  - 20000-20007 
    # mailing  - 21000-21007
    # location - 22000-22007
    #
    # to add a new address category, just add the name
    # to self.keys and give the elements IDs with
    # 20000+(x*1000), where x is the position of the
    # string in self.keys

    if type == 'person': self.keys = getAddAddressKeys()
    else: self.keys = getAddEnterpriseAddressKeys()
    self.address = getAddAddressFields()
    self.phoneinfo = getAddPhoneInfoFields()
    self.salutation = getAddViewSalutationEntries()
    self.fields = getAddTextFields()
 
    self.fillChoice(ID_CHO_GENDER,self.gender)
    self.fillChoice(ID_CHO_SALUTATION,self.salutation,0) 

    list = self.getListCtrl(ID_LST_EXTATTR)
    list.InsertColumn(0,'Attribute')
    list.SetColumnWidth(0,165)
    list.InsertColumn(1,'Value')
    list.SetColumnWidth(1,220)

    self.extAttr = getMainExtAttr()

    # fill the listCtrl for the extended attributes 
    for i in range(len(self.app.values)):
      if self.extAttr.has_key(self.app.values[i]['key']):
        list.InsertStringItem( i,self.extAttr[self.app.values[i]['key']])
      else:
        list.InsertStringItem(i,self.app.values[i]['key'])

    # check if we have a record number in the constructor
    # if yes, fill all the controls with values
    if record != None:

      # fetch the record belonging to the number
      self.result = self.app.elementList[record]

      if type == 'person':
        self.SetTitle('Editing person (' + self.result['name'] + ', ' +
                      self.result['firstname'] + ')')
      else:
        self.SetTitle('Editing enterprise ' + self.result['name'])

      # textfields
      for count in range(len(self.fields)):
        field = self.fields[count]
        self.setText(field[0],self.result[field[1]])

      # salutation
      if self.result.has_key('salutation'):
        for count in range(len(self.salutation)):
          if self.salutation[count][1] == self.result['salutation']:
            self.getChoice(ID_CHO_SALUTATION).SetStringSelection(
                                              self.salutation[count][0])

      # sex/gender
      if self.result.has_key('sex'):
        self.getChoice(ID_CHO_GENDER).SetStringSelection(self.result['sex'])
      
      # birthday
      if self.result.has_key('birthday'):
        self.setText(ID_TXT_BIRTHDAY,self.result.getBirthday())
 
      # phone
      if self.result.has_key('phones'):
        if self.result['phones'] != {}:
          phoneDict = self.result['phones']
          for count in range(len(self.phone)):
            if phoneDict.has_key(self.phone[count][1]):
              self.setText(self.phone[count][0],
                            str(phoneDict[self.phone[count][1]]['number']))
              self.setText(self.phoneinfo[count],
                                str(phoneDict[self.phone[count][1]]['info']))

      # addresses 
      addresses = self.result['addresses']
    
      for key in range(len(self.keys)):
        if addresses.has_key(self.keys[key]):
          theAddress = addresses[self.keys[key]]
  
          for count in range(len(self.address)):
            if theAddress.has_key(self.address[count][1]):
              self.setText(self.address[count][0]+(key*1000),
                           theAddress[self.address[count][1]])

      # extended attributes
      if self.result.has_key('extendedAttrs'):
        extendedAttrs = self.result['extendedAttrs']
        list.DeleteAllItems()
        count = 0
        for key in self.result['extendedKeys']:
          if extendedAttrs.has_key(key):
            if self.extAttr.has_key(key):
              list.InsertStringItem(count,self.extAttr[key])
            else:    
              list.InsertStringItem(count,key)
            list.SetStringItem(count,1,extendedAttrs[key])
            count = count +1

    else:
      server = self.app.connectionList[type]
      if type == 'person': self.result = Person(server)
      else: self.result = Enterprise(server)

    # WDR: handler declarations for wxDlgAddEditPerson

    EVT_BUTTON(self, ID_BTN_CALENDAR, self.onCalendar)
    EVT_BUTTON(self, ID_BTN_ADD, self.onAdd)
    EVT_BUTTON(self, ID_BTN_CANCEL, self.onCancel)
    EVT_BUTTON(self, ID_BTN_CONTACT, self.onContact)
  
    EVT_LIST_ITEM_ACTIVATED(list, -1, self.onItemActivated)
    EVT_LIST_ITEM_RIGHT_CLICK(list, -1, self.onRightClick)

  # WDR: handler implementations for wxDlgAddEditPerson

  # onContact
  # -> find a contact
  def onContact(self,event):
    text = self.getText(ID_TXT_CONTACT)
    if text != '':
      dlg = wxDlgGetContact(self, -1, 'Select Contact', text, self.app)
      if dlg.ShowModal() == wxID_OK:
        self.setText(ID_TXT_CONTACT,dlg.getResult())
      dlg.Destroy()

  # onRightClick
  # -> show popupmenu
  def onRightClick(self,event):
    menu = wxMenu()

    menu.Append(6,"Edit")
    EVT_MENU(self, 6, self.onItemEdit)
    menu.Append(7,"Delete")
    EVT_MENU(self, 7, self.onItemDelete)
    menu.Append(8,"Add")
    EVT_MENU(self,8,self.onItemAdd)

    self.getListCtrl(ID_LST_EXTATTR).PopupMenu(menu, event.GetPoint())
    menu.Destroy()

  # onItemEdit
  # -> edit the value of an extended attribute
  def onItemEdit(self,event):
    self.onItemActivated(-1)    

  # onItemDelete
  # -> delete an extended attribute
  def onItemDelete(self,event):
    list = self.getListCtrl(ID_LST_EXTATTR)
    item = list.GetNextItem( -1,wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)
    list.DeleteItem(item) 
   
  # onItemAdd
  # -> add an extended attribute to the record
  def onItemAdd(self,event):
    dlg = wxTextEntryDialog(self,'Enter name of the new attribute','Edit Name','')
    list = self.getListCtrl(ID_LST_EXTATTR)
    if dlg.ShowModal() == wxID_OK:
      dlg2 = wxTextEntryDialog(self,'Enter the value',
                                    'Edit Value of %s' % dlg.GetValue(),'')
      if dlg2.ShowModal() == wxID_OK:
        if not self.result.has_key('extendedKeys'):
          self.result['extendedKeys'] = []
        self.result['extendedKeys'].append(dlg.GetValue())
        if not self.result.has_key('extendedAttrs'):
          self.result['extendedAttrs'] = {}
        self.result['extendedAttrs'][dlg.GetValue()] = dlg2.GetValue()
        list.InsertStringItem(list.GetItemCount(),dlg.GetValue())
        list.SetStringItem(list.GetItemCount()-1,1,dlg2.GetValue())

      dlg2.Destroy()
    dlg.Destroy()

  # onItemActivated
  # -> edit the current listCtrl Entry
  def onItemActivated(self,event):
    list = self.getListCtrl(ID_LST_EXTATTR)
    itemName = list.GetItemText(list.GetNextItem(
                                -1,wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED))

    for key in self.extAttr.keys():
      if self.extAttr[key] == itemName:
        itemName = key

    editString = ''

    if self.result.has_key('extendedAttrs'):
      if self.result['extendedAttrs'].has_key(itemName):
        editString = self.result['extendedAttrs'][itemName]

    if self.extAttr.has_key(itemName):
      editText = self.extAttr[itemName]
    else: editText = itemName

    dlg = wxTextEntryDialog(self,editText,'Edit Value',editString)
    if dlg.ShowModal() == wxID_OK:
      list.SetStringItem(event.m_itemIndex,1,dlg.GetValue())
      if not self.result.has_key('extendedAttrs'):
        self.result['extendedAttrs'] = {}
        self.result['extendedKeys'] = self.extAttr.keys()
      self.result['extendedAttrs'][itemName] = dlg.GetValue()

  # onCalendar
  # -> select a date from the calendar
  def onCalendar(self, event):
    dlg = wxDlgCalendarSelect(self, -1, 'Select Date', self.getText(ID_TXT_BIRTHDAY))
    if dlg.ShowModal() == wxID_OK:
      self.setText(ID_TXT_BIRTHDAY,dlg.getResult())
    dlg.Destroy()

  # onAdd
  # -> add a person to the SKYRiX system
  def onAdd(self,event):
    # textfields
    for count in range(len(self.fields)):
      field = self.fields[count]
      self.result[field[1]] = self.getText(field[0])

    # salutation
    self.result['salutation'] = self.elemChoice(ID_CHO_SALUTATION,self.salutation,1)
    
    # sex/gender
    if self.getChoice(ID_CHO_GENDER) != None:
      if self.getChoice(ID_CHO_GENDER).GetSelection() != 0:
        self.result['sex'] = self.getChoice(ID_CHO_GENDER).GetStringSelection()

    # birthday
    if self.type == 'person':
      self.result.setBirthday(self.getText(ID_TXT_BIRTHDAY))
  
    # phones
    for i in range(len(self.phone)):
      self.result.setPhone(self.phone[i][1],self.getText(self.phone[i][0]),
                                            self.getText(self.phoneinfo[i]))

    # addresses
    addressDict = {}    
    tempDict = {}
   
    for key in range(len(self.keys)):
      tempDict['type'] = self.keys[key]
      for count in range(len(self.address)):
        if self.getText(self.address[count][0]+(key*1000)) != '':
          tempDict[self.address[count][1]] = self.getText(
                   self.address[count][0]+(key*1000))
      addressDict[self.keys[key]] = tempDict
      tempDict = {}    

    self.result['addresses'] = addressDict

    self.result.saveObject()
   
    if self.app.appType == 'person':
       self.app.frame.windowPerson.initWithFetchResults(self.app.elementList,
                    self.app.frame.windowPerson.getListCtrl(ID_LCT_NAMES))
    if self.app.appType == 'enterprise':
       self.app.frame.windowEnterprise.initWithFetchResults(self.app.elementList,
                      self.app.frame.windowEnterprise.getListCtrl(ID_LCT_NAMES))
    self.Close()

  # onCancel
  # -> cancel adding/editing/viewing
  def onCancel(self,event):
    self.Close()
    return 1

