#!/usr/bin/env python
#----------------------------------------------------------------------------
# Name:         wxDlgMain.py
# Author:       Bjoern
# Created:      09/07/2001
# Copyright:    (c) 2001 SKYRiX Software AG
#----------------------------------------------------------------------------

from wxPython.wx import *
from wxSkyClient_wdr import dlgMain, dlgEnterpriseMain
from wxDlgViewDocument import wxDlgViewDocument
from wxSkyData import *

class wxDlgMain(wxPanel):
  
  # __init__
  def __init__(self, parent, id = -1, app = None, pos = (0,0), 
               size = wxPyDefaultSize, style = wxRESIZE_BORDER|wxCAPTION|wxSYSTEM_MENU):

    wxPanel.__init__(self, parent, id, pos, size)

    self.app = app
    self.appType = app.appType

    self.fields = getMainFields()
    self.phone =  getMainPhones()
    self.extAttr = getMainExtAttr()
    self.maxres = getMainMaxRes()
    self.names = getSkyCategories()
   
    self.page = 0
    self.sortOrdering = ('number', true)

    # create the main frame, fill it with the resource
    if self.appType == 'person':
      dlgMain(self, true)
    else: dlgEnterpriseMain(self,true)
  
    self.CentreOnScreen()

    categoryChoice = self.getChoice(ID_CHO_CATEGORY)
    otherChoice = self.getChoice(ID_CHO_OTHER)

    # ini phone and maxres choice
    self.fillChoice(ID_CHO_PHONE,self.phone,0)
    self.fillChoice(ID_CHO_RESULTLIMIT,self.maxres)

    if categoryChoice != None:
      categoryChoice.Append('-')
      categoryChoice.SetSelection(0)
    if otherChoice != None:
      otherChoice.Append('-')
      otherChoice.SetSelection(0)

    EVT_BUTTON(self,ID_BTN_FIND,self.onSearch)
    EVT_BUTTON(self,ID_BTN_SEARCH,self.onEnhancedFind)
    EVT_BUTTON(self,ID_BTN_FULLTEXT,self.onFullTextSearch)
    EVT_BUTTON(self,ID_BTN_CLEAR,self.onClear)
    EVT_LIST_ITEM_ACTIVATED(self,ID_LCT_NAMES,self.onItemActivated)
    EVT_NOTEBOOK_PAGE_CHANGED(self,ID_NTB_MAIN,self.onNotebookChanged)

  def getListForPage(self,page):
    if page == 0:
      return wxPyTypeCast( self.FindWindowById(ID_LCT_NAMES), "wxListCtrl" )
    else:
      return wxPyTypeCast( self.FindWindowById(ID_LCT_NAMES2), "wxListCtrl" )
  
  # initPanel
  # -> init the application after __init__
  def initPanel(self, app):

    self.app = app

    comboBox1 = self.getComboBox(ID_CMB_SEARCH)
    comboBox2 = self.getComboBox(ID_CMB_SEARCH2)

    # set column widths
    self.initList(self.getListForPage(0))
    self.initList(self.getListForPage(2))

    # init the comboBox with values from the config
    if self.appType == 'person':
      searchdefs = app.config.getValue('browser.searchdefs')
    else:
      searchdefs = app.config.getValue('enterprise.defs')
    if searchdefs != '':
      for count in range(len(searchdefs)):
        comboBox1.Append(searchdefs[count])
        comboBox2.Append(searchdefs[count])
      comboBox1.SetSelection(0)
      comboBox2.SetSelection(0)

  # initExtAttrList
  # initialize the extAttrs listCtrl
  def initExtAttrList(self):
    other = self.getChoice(ID_CHO_OTHER)

    server = self.app.connectionList['ping']

    try:
      if self.appType == 'person':
        self.app.values = server.defaults.objectForKey(
                  'SkyPublicExtendedPersonAttributes')
      else:
         self.app.values = server.defaults.objectForKey(
                  'SkyPublicExtendedEnterpriseAttributes')
      for i in range(len(self.app.values)):
        if self.extAttr.has_key(self.app.values[i]['key']):
          other.Append(self.extAttr[self.app.values[i]['key']])
        else:
          other.Append(self.app.values[i]['key'])
      self.app.isConnected = true
    except:
      dlg = wxMessageDialog(self, 'Connection to host failed !', 'Connect Error',
                            wxOK|wxICON_ERROR)
      self.app.isConnected = false
      dlg.ShowModal()

  # initList
  def initList(self,list):
    self.oldCol = -1
    self.sortAsc = true

    if self.appType == 'person': self.cols  = self.app.config.getValue('list.keys')
    else: self.cols  = self.app.config.getValue('enterprise.keys')
    self.keys   = []
       
    for count in range(len(self.cols)):
      self.keys.append(self.cols[count][0])

    self.keys = tuple(self.keys)

    # results from fetch (the stuff that is shown in the listCtrl)
    self.results = []

    # set the event handlers
    EVT_LIST_ITEM_ACTIVATED(list,-1,self.onItemActivated)
    EVT_LIST_ITEM_SELECTED(list,-1,self.onItemSelected)
    EVT_LIST_ITEM_RIGHT_CLICK(list,-1,self.onRightClick)
    EVT_LIST_COL_CLICK(list,-1,self.onColClick)

    # set columns and columnwidth
    self.columnCount = 0
    self.reInitColumns(list,self.keys)

    for count in range(len(self.cols)):
      list.SetColumnWidth(count+1,self.cols[count][1])

    list.SetColumnWidth(0,100)

  # initWithFetchResults
  # -> init the tableview with results from a fetchObjects()
  def initWithFetchResults(self,results, list):
    self.app.elementList = results
    self.results = results
    list.DeleteAllItems()
    counter = 0
    for p in self.results.keys():

      result = self.results[p]

      if result.has_key('extendedAttrs'): extAttr = result['extendedAttrs']
      else: extAttr = []
      if result.has_key('phones'): phones = result['phones']
      else: phones = {}

      if len(self.results) > 1: record = 'records'
      else: record = 'record'

      #self.target.SetStatusText('Found %s matching %s' % ( len(self.results),record),0)
      #self.target.SetStatusText('',1)

      list.InsertStringItem(counter,str(result[self.keys[0]]))

      # used for sorting the columns
      list.SetItemData(counter,counter)

      for count in range(1,len(self.keys)): 

        # 'normal' keys
        if result.has_key(self.keys[count]):
          list.SetStringItem(counter,count,result[self.keys[count]])
        
        # extended attributes
        if extAttr != []:
          if extAttr.has_key(self.keys[count]):
            list.SetStringItem(counter,count,extAttr[self.keys[count]])

        # phone
        if phones != {}:
          if phones.has_key(self.keys[count]):
            if phones[self.keys[count]]['number'] != '\012':
              addItem = phones[self.keys[count]]['number'] + ' (' +  \
                        phones[self.keys[count]]['info'] + ')'
              list.SetStringItem(counter,count,addItem)
      counter = counter + 1

  # reInitColumns
  # -> set new columns
  def reInitColumns(self,list,keys = None):

    if keys != None:
      # 'number' is always the first column (we need smth to sort by)
      self.keys = ("number",) + tuple(keys)

    # delete all items
    list.ClearAll()

    # add new columns
    for p in range(len(self.keys)):
      for i in range(len(self.names)):
        if self.names[i][0] == self.keys[p]:
          list.InsertColumn(p,self.names[i][1])
          break 
      if self.extAttr.has_key(self.keys[p]):
        list.InsertColumn(p,self.extAttr[self.keys[p]])
      for i in range(len(self.phone)):
        if self.keys[p] == self.phone[i][1]:
          list.InsertColumn(p,self.phone[i][0])
          break

    if self.columnCount != 0: self.initWithFetchResults(self.app.elementList, list)
    self.columnCount = len(self.keys)

  # onNotebookChanged
  # -> notebook view changed
  def onNotebookChanged(self,event):
    self.page = event.GetSelection()

  # onFindEnhanced
  # -> search enhanced more
  def onEnhancedFind(self, event):
    if self.app.checkIfConnected(self.appType):
 
      server = self.app.connectionList[self.appType]
      server.setFetchSpecification('')

      # 'normal' fields
      for count in range(len(self.fields)):
        field = self.fields[count]
        server.addToFetchSpecification(self.getText(field[0]), [field[1],])
      
      # phone number
      choice = self.getChoice(ID_CHO_PHONE).GetSelection()
      if choice != 0:
        server.addToFetchSpecification(self.phone[choice][1], ['phone.type',])
      server.addToFetchSpecification(self.getText(ID_TXT_PHONE), ['phone.number',])
   
      # extended attributes
      theName = self.app.values[self.getChoice(ID_CHO_OTHER).GetSelection()-1]['key']
      server.addToFetchSpecification(self.getText(ID_TXT_OTHER), [theName,])
   
      _fs = server.fetchSpecification
      print _fs
      if _fs  != '':
        fs = {}
        fs['qualifier'] = _fs
        fs['fetchLimit'] = int(self.getChoice(ID_CHO_RESULTLIMIT).GetStringSelection())

        server.setFetchSpecification(_fs)
        self.result = server.fetchObjects()
        if self.result != []:
          self.initWithFetchResults(self.result,self.getListForPage(0))
          self.getNotebook(ID_NTB_MAIN).SetSelection(0)
        else:
          dlg = wxMessageDialog(self,'No entry found','Search Error',
                                wxOK|wxICON_ERROR)
          dlg.ShowModal()
      else:
        dlg = wxMessageDialog(self,'No search criteria entered.',
                              'Search Error', wxOK|wxICON_ERROR)
        dlg.ShowModal()

  # onClear
  # -> clear the form
  def onClear(self,event):
    for count in range(len(self.fields)):
      field = self.fields[count]
      getCtrl(self,field[0],'wxTextCtrl').Clear()

  # onSearch
  # -> find a person
  def onSearch(self,event):
    if self.app.checkIfConnected(self.appType):
      comboBox = self.getComboBox(ID_CMB_SEARCH)
      text = comboBox.GetValue()
      
      if comboBox.FindString(text) == -1:
        comboBox.Append(text)
      comboBox.SetSelection(comboBox.FindString(text))
  
      if text:
        if self.appType == 'person':
          keys = ['name','nickname','firstname','login']
        else: keys = ['name','number']
        
        server = self.app.connectionList[self.appType]
        server.addToFetchSpecification(text, keys,'OR')
        self.initWithFetchResults(server.fetchObjects(), self.getListForPage(0))

      else:
        #FIXME : appName ??
        wxDlgMsgBox(self, 'No Search criteria entered, try again !', appName)

  # onFullTextSearch
  # -> find a person with fulltext search
  def onFullTextSearch(self,event):
    if self.app.checkIfConnected(self.appType):
      comboBox = self.getComboBox(ID_CMB_SEARCH2)
      text = comboBox.GetValue()
      comboBox.Append(text)
  
      if text:
        server = self.app.connectionList[self.appType]
        server.addToFetchSpecification(text,['fullSearchString',])
        self.initWithFetchResults(server.fetchObjects(), self.getListForPage(2))
      else:
        wxDlgMsgBox(self, 'No Search criteria entered, try again !', appName)

  # onItemSelected
  # -> show which item is selected
  def onItemSelected(self,event):  
    list = self.getListForPage(self.page)
    item = list.GetItem(list.GetNextItem(-1,wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED))

  # onItemActivated
  # -> edit item from listCtrl
  def onItemActivated(self,event):
    if self.app.checkIfConnected(self.appType):
      list = self.getListForPage(self.page)
      theName = list.GetItemText(list.GetNextItem(-1,wxLIST_NEXT_ALL, 
                                                  wxLIST_STATE_SELECTED))
      frame = wxDlgViewDocument(self,-1,'View person ' + theName,self.app,theName,self.app.appType)
      frame.Show(true)

  # onRightClick
  # -> show popupmenu
  def onRightClick(self,event):
    menu = wxMenu()

    menu.Append(0,"Edit")
    EVT_MENU(self, 0, self.app.frame.onMenuPersonEdit)
    menu.Append(1,"Delete")
    EVT_MENU(self, 1, self.app.frame.onMenuPersonDelete)
    menu.Append(2,"View")
    EVT_MENU(self,2,self.app.frame.onMenuPersonView)

    list = self.getListForPage(self.page)
    list.PopupMenu(menu, event.GetPoint())
    menu.Destroy()

  # onColClick
  # selecting a column for sorting
  def onColClick(self,event):
  
    self.col = event.m_col
    key = self.keys[self.col]
    if self.sortAsc: asc = 'ascending'
    else: asc = 'descending'

    if self.sortAsc:
      self.sortOrdering = (key, true)
    else:
      self.sortOrdering = (key, false)

    list = self.getListForPage(self.page)

    if self.col == self.oldCol:
      if self.sortAsc == true:
      	list.SortItems(self.columnSorter)
      	self.sortAsc = false
      else:
      	list.SortItems(self.columnSorter)
        self.sortAsc = true
    else:
      list.SortItems(self.columnSorter)
      self.sortAsc = false
    self.oldCol = self.col

  # columnSorter
  # -> sort selected column
  def columnSorter(self, key1, key2):
    if self.sortAsc:
      returnValue = 1
    else:
      returnValue = -1

    item1 = ''
    item2 = ''

    # item 1
    if self.results[key1].has_key(self.keys[self.col]):
      item1 = self.results[key1][self.keys[self.col]]
    if self.results[key1].has_key('extendedAttrs'):
      if self.results[key1]['extendedAttrs'].has_key(self.keys[self.col]):
        item1 = self.results[key1]['extendedAttrs'][self.keys[self.col]]
    if self.results[key1].has_key('phones'):
      if self.results[key1]['phones'].has_key(self.keys[self.col]):
        item1 = self.results[key1]['phones'][self.keys[self.col]]['number']

    # item 2
    if self.results[key2].has_key(self.keys[self.col]):
      item2 = self.results[key2][self.keys[self.col]]
    if self.results[key2].has_key('extendedAttrs'):
      if self.results[key2]['extendedAttrs'].has_key(self.keys[self.col]):
        item2 = self.results[key2]['extendedAttrs'][self.keys[self.col]]
    if self.results[key2].has_key('phones'):
      if self.results[key2]['phones'].has_key(self.keys[self.col]):
        item2 = self.results[key2]['phones'][self.keys[self.col]]['number']

    if item1 == item2:  return 0
    elif item1 < item2: return -returnValue
    else:               return returnValue

