#!/usr/bin/env python
#----------------------------------------------------------------------------
# Name:         wxDlgSelectColumns.py
# Author:       Bjoern
# Created:      09/25/2001
# Copyright:    (c) 2001 SKYRiX Software AG
#----------------------------------------------------------------------------

from wxPython.wx import *
from wxSkyClient_wdr import dlgSelectColumns, ID_LIB_NAMES
from skyrix.ui.Dialog import *
from wxSkyData import *

class wxDlgSelectColumns(wxDialog):

  # __init__
  def __init__(self, parent, id, title,keys,type,
               pos = wxPyDefaultPosition, size = wxPyDefaultSize,
               style = wxDEFAULT_DIALOG_STYLE ):

    wxDialog.__init__(self, parent, id, title, pos, size, style)

    wxInitAllImageHandlers()
    dlgSelectColumns( self, true )
    self.CentreOnParent()

    self.result  = []

    listBox = self.getListBox(ID_LIB_NAMES)

    self.categoriesList = getSkyCategories()
    self.extAttr = getMainExtAttr()
    self.phones = getMainPhones()

    # add common attributes
    for count in range(len(self.categoriesList)):
      if self.categoriesList[count][0] != 'number':
        listBox.Append(self.categoriesList[count][1])

    # add extended attributes
    for i in self.extAttr.keys():
      listBox.Append(self.extAttr[i])
    
    # add phones
    for count in range(len(self.phones)):
       if self.phones[count][1] != '':
         listBox.Append(self.phones[count][0])

    theString = None

    # check already selected columns
    for key in keys:
      for i in range(len(self.categoriesList)):
        if key == self.categoriesList[i][0]:
          index = listBox.FindString(self.categoriesList[i][1])
          if index != -1: listBox.SetSelection(index)
          break
      for i in range(len(self.phones)):
        if key == self.phones[i][1]:
          index = listBox.FindString(self.phones[i][0])
          if index != -1: listBox.SetSelection(index)
          break
      for i in self.extAttr.keys():
        if key == i:
          index = listBox.FindString(self.extAttr[key])
          if index != -1: listBox.SetSelection(index)
          break

    # WDR: handler declarations for wxDlgSelectColumns

    EVT_BUTTON(self , ID_BTN_SELECT   , self.onSelect)
    EVT_BUTTON(self , ID_BTN_TAKE_SEL , self.onTakeSelection)
    EVT_BUTTON(self , ID_BTN_UP       , self.onUp)
    EVT_BUTTON(self , ID_BTN_DOWN     , self.onDown)
    EVT_LISTBOX(self, ID_LIB_ORDER    , self.onListItem)

  # WDR: methods for wxDlgSelectColumns

  # getResult
  # -> return the list of selected columns
  def getResult(self):
    return tuple(self.result)

  # WDR: handler implementations for wxDlgSelectColumns

  # onListItem
  # -> set buttons depending on listitem position
  def onListItem(self,event):
    order = self.getListBox(ID_LIB_ORDER)
    item = order.GetSelection()
    btnUp = wxPyTypeCast(self.FindWindowById(ID_BTN_UP), 'wxBitmapButton')
    btnDown = wxPyTypeCast(self.FindWindowById(ID_BTN_DOWN), 'wxBitmapButton')

    if item == 0: btnUp.Enable(false)
    else: btnUp.Enable(true)

    if item == order.Number()-1: btnDown.Enable(false)
    else: btnDown.Enable(true)

  # onTakeSelection
  # -> take selection
  def onTakeSelection(self,event):
    listBox = self.getListBox(ID_LIB_NAMES)
    orderBox = self.getListBox(ID_LIB_ORDER)
    for count in listBox.GetSelections():
      theString = listBox.GetString(count)
      for i in range(len(self.categoriesList)):
        if self.categoriesList[i][1] == theString:
          orderBox.Append(self.categoriesList[i][1])
      for i in range(len(self.phones)):
        if self.phones[i][0] ==  theString:
          orderBox.Append(self.phones[i][0])
      for i in self.extAttr.keys():
        if theString == self.extAttr[i]:
          orderBox.Append(self.extAttr[i])

    wxPyTypeCast(self.FindWindowById(ID_BTN_DOWN), 'wxBitmapButton').Enable(true)
    wxPyTypeCast(self.FindWindowById(ID_BTN_TAKE_SEL), 'wxBitmapButton').Enable(true)

  # onUp
  # -> move items up in the orderList
  def onUp(self,event):
    order = self.getListBox(ID_LIB_ORDER)
    item = order.GetSelection() 
    
    theText = order.GetString(item-1)
    order.SetString(item-1,order.GetString(item))
    order.SetString(item,theText)
    order.SetSelection(item-1)
    self.onListItem(-1)

  # onDown
  # -> move items down in the orderList
  def onDown(self,event):
    order = self.getListBox(ID_LIB_ORDER)
    item = order.GetSelection() 
    
    theText = order.GetString(item+1)
    order.SetString(item+1,order.GetString(item))
    order.SetString(item,theText)
    order.SetSelection(item+1)
    self.onListItem(-1)

  # onSelect
  # -> get the chosen columns from the ListBox
  def onSelect(self,event):
    order = self.getListBox(ID_LIB_ORDER)
    for count in range(order.Number()):
      for i in range(len(self.categoriesList)):
        if self.categoriesList[i][1] == order.GetString(count):
          self.result.append(self.categoriesList[i][0]) 
      for i in range(len(self.phones)):
        if self.phones[i][0] == order.GetString(count):
          self.result.append(self.phones[i][1])
      for i in self.extAttr.keys():
        if self.extAttr[i] == order.GetString(count):
          self.result.append(i)

    self.EndModal(wxID_OK)
