#!/usr/bin/env python
#----------------------------------------------------------------------------
# Name:         wxDlgSkyClient.py
# Author:       bjoern
# Created:      09/24/2001
# Copyright:    (c) 2001 SKYRiX Software AG
#----------------------------------------------------------------------------

from wxCustomStatusBar import wxCustomStatusBar
from wxDlgAbout import wxDlgAbout
from wxPython.wx import *
from wxSkyClient_wdr import *
from skyrix import *

from wxDlgMain import wxDlgMain
from wxDlgAddEditDocument import wxDlgAddEditDocument
from wxDlgBusinessCardGather import wxDlgBusinessCardGather
from wxDlgViewDocument import wxDlgViewDocument
from wxDlgSelectColumns import wxDlgSelectColumns
from wxDlgMakeConnection import wxDlgMakeConnection
from wxPanProjectBrowser import wxPanProjectBrowser

# wxDlgSkyClient
class wxDlgSkyClient(wxFrame):

  # __init__
  def __init__(self, parent, id, app):
    self.app = app

    wxInitAllImageHandlers()
    wxFrame.__init__(self, parent, id, 'wxSkyClient - Alpha',size=(900,700))

    self.splitter = wxSplitterWindow(self, id)
  
    self.windowPerson = None
    self.windowEnterprise = None
    self.windowProjects = None

    # menu
    self.window1 = wxPanel(self.splitter, -1, style=wxVSCROLL)

    colour = wxColour(120,120,120)
    
    self.window1.SetBackgroundColour(wxLIGHT_GREY)
  
    buttons = [(ID_BTN_CONTACTS,6), (ID_BTN_ENTERPRISES,5), (ID_BTN_PROJECTS,8)]
  
    sizer = wxFlexGridSizer(0,1,0,0)
    sizer.AddGrowableCol(0)
 
    button = wxButton(self.window1, ID_BTN_OK, 'About')
    button.SetBackgroundColour(colour)
    sizer.AddWindow(button, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 0)

    for element in buttons:
      button = wxBitmapButton(self.window1, element[0], bmpList(element[1]),
                              style=wxNO_BORDER)
      sizer.AddWindow(button, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5)

    self.window1.SetAutoLayout(true)
    self.window1.SetSizer(sizer)
    sizer.Fit(self.window1)
    sizer.SetSizeHints(self.window1)

    # 'main' enterprise window
    self.windowPerson = wxPanel(self.splitter, -1)
    # self.windowPerson.initPanel(self.app)

    # splitter window
    self.splitter.SetMinimumPaneSize(90)
    self.splitter.SplitVertically(self.window1, self.windowPerson)
    self.splitter.SetSashPosition(90)

    # statusBar
    self.statusBar = wxCustomStatusBar(self)
    self.SetStatusBar(self.statusBar)
    self.SetStatusText('wxSkyClient - Alpha',0)
  
    # menuBar
    self.SetMenuBar(wxDlgSkyClientMenu())
  
    # events
    EVT_BUTTON(self,ID_BTN_CONTACTS,self.onButtonContacts)
    EVT_BUTTON(self,ID_BTN_ENTERPRISES,self.onButtonEnterprise)
    EVT_BUTTON(self,ID_BTN_PROJECTS, self.onButtonProjects)

    # connection menu
    EVT_MENU(self, ID_SUB_CONNECT, self.onMenuConnect)
    EVT_MENU(self, ID_SUB_DISCONNECT, self.onMenuDisconnect)
    EVT_MENU(self, ID_SUB_EXIT, self.onMenuExit)

    # add menu
    EVT_MENU(self, ID_SUB_NEWPERSON, self.onMenuPersonAdd)
    EVT_MENU(self, ID_SUB_BUSINESSCARD, self.onMenuPersonBusiness)
    EVT_MENU(self, ID_SUB_NEWENTERPRISE, self.onMenuEnterpriseAdd)
    
    # config menu
    # FIXME: handle config columns and stuff
    EVT_MENU(self, ID_SUB_COLUMNS, self.onMenuSetColumns)

    # help menu
    EVT_MENU(self, ID_SUB_ABOUT, self.onMenuHelpAbout)
    
    self.CentreOnScreen()
    self.Show(true)

  # onMenuConnect
  # -> open dialog for connection to a new server
  def onMenuConnect(self,event):
    dlgConnect = wxDlgMakeConnection(self, -1, 'Connect to an XMLRPC Server', self.app,                                    self.app.appType)
    dlgConnect.ShowModal()
    dlgConnect.Destroy()

  # onMenuDisconnect
  # -> disconnect from a server
  def onMenuDisconnect(self, event):
    if self.app.appType in ['person','enterprise']:
      del self.app.connectionList['person']
      del self.app.connectionList['enterprise']
      del self.app.connectionList['ping']
    else:
      del self.app.connectionList[self.app.appType]

  # onMenuExit
  # -> quit program
  def onMenuExit(self, event):
    self.Destroy()
    self.OnExit()

  # onMenuPersonBusiness
  # -> business card gathering
  def onMenuPersonBusiness(self,event):
    if self.app.checkIfConnected('person'): 
      if not self.app.connectionList.has_key('person'):
        dlg = wxDlgMakeConnection(self,-1,'Setup Contact connection',self.app,'person')
        if dlg.ShowModal() == wxID_OK:
          dlg.Destroy()
    frame = wxDlgBusinessCardGather(self, -1, 'Business card gathering',self.app)
    frame.Show(true)

  # onMenuPersonAdd
  # -> add a person to the SKYRiX system
  def onMenuPersonAdd(self,event):
    if self.app.checkIfConnected('person'): 
      if not self.app.connectionList.has_key('person'):
        dlg = wxDlgMakeConnection(self,-1,'Setup Contact connection',self.app,'person')
        if dlg.ShowModal() == wxID_OK:
          dlg.Destroy()
      frame = wxDlgAddEditDocument(self, -1, 'Add person',self.app,None,'person')
      frame.Show(true)
 
  # onMenuEnterpriseAdd
  # -> add an enterprise to the SKYRiX system
  def onMenuEnterpriseAdd(self,event):
    if self.app.checkIfConnected('enterprise'): 
      if not self.app.connectionList.has_key('enterprise'):
        dlg = wxDlgMakeConnection(self,-1,'Setup Enterprise connection',self.app,'enterprise')
        if dlg.ShowModal() == wxID_OK:
          dlg.Destroy()
      frame = wxDlgAddEditDocument(self, -1, 'Add enterprise',self.app,None,'enterprise')
      frame.Show(true)
 
  # onMenuPersonEdit
  # -> edit the entry which is selected
  def onMenuPersonEdit(self,event):
    if self.app.checkIfConnected('person'): 
      list = self.getListCtrl(ID_LCT_NAMES)
      theName = list.GetItemText(list.GetNextItem(-1,wxLIST_NEXT_ALL,
                                              wxLIST_STATE_SELECTED))
      frame = wxDlgAddEditDocument(self,-1,'Edit person',self.app, theName,'person')
      frame.Show(true)

  # onMenuPersonView
  # -> edit the entry which is selected
  def onMenuPersonView(self,event):
    if self.app.checkIfConnected('person'): 
      list = self.getListCtrl(ID_LCT_NAMES)
      theName = list.GetItemText(list.GetNextItem(-1,wxLIST_NEXT_ALL,
                                              wxLIST_STATE_SELECTED))
      frame = wxDlgViewDocument(self, -1, 'View person',self.app,theName,'person')
      frame.Show(true)

  # onMenuPersonDelete
  # -> delete person
  def onMenuPersonDelete(self,event):
    if self.app.checkIfConnected('person'):

      list = self.getListCtrl(ID_LCT_NAMES)
      theName = list.GetItemText(list.GetNextItem(-1,wxLIST_NEXT_ALL, 
                                                  wxLIST_STATE_SELECTED))

      self.app.elementList[theName].deleteObject()

  # onMenuSetColumns
  # -> change columns of the listCtrl
  def onMenuSetColumns(self,event):
    if self.app.appType == 'person' or self.app.appType == 'enterprise':
      if self.app.appType == 'person':
        keys = self.app.frame.windowPerson.keys
      elif self.app.appType == 'enterprise':
        keys = self.app.frame.windowEnterprise.keys

      dlgSelect = wxDlgSelectColumns(self, -1, 'Select columns for the main list',
                                    keys,self.app.appType)
      if dlgSelect.ShowModal() == wxID_OK:
        notebook = self.getNotebook(ID_NTB_MAIN)
        notebook.SetSelection(1)
        if self.app.appType == 'person':
          self.windowPerson.reInitColumns(self.getListCtrl(ID_LCT_NAMES),
                                          dlgSelect.getResult())
          self.windowPerson.reInitColumns(self.getListCtrl(ID_LCT_NAMES2),
                                          dlgSelect.getResult())
        if self.app.appType == 'enterprise':
          self.windowEnterprise.reInitColumns(self.getListCtrl(ID_LCT_NAMES),
                                          dlgSelect.getResult())
          self.windowEnterprise.reInitColumns(self.getListCtrl(ID_LCT_NAMES2),
                                          dlgSelect.getResult())
        notebook.SetSelection(0)
      dlgSelect.Destroy()

  # onMenuHelpAbout
  # -> about screen
  def onMenuHelpAbout(self,event):
    dlg = wxDlgAbout(self, -1, 'About wxSkyPersonBrowser')
    dlg.ShowModal() 
 
  # onButtonContacts
  def onButtonContacts(self,event):
    
    start = true
    self.app.appType = 'person'
    if not self.app.connectionList.has_key('person'):
      dlg = wxDlgMakeConnection(self,-1,'Setup Contacts connection',self.app,'person')
      if dlg.ShowModal() != wxID_OK:
        start=false
      dlg.Destroy()
    if start:
      self.windowPerson = wxDlgMain(self.splitter, -1, self.app)
      self.windowPerson.initPanel(self.app)
      self.windowPerson.initExtAttrList()
      window = self.splitter.GetWindow2()
      self.splitter.ReplaceWindow(window, self.windowPerson)
      window.Destroy()
      self.SetStatusText('Contacts application started ....',0)

  # onButtonEnterprise
  def onButtonEnterprise(self,event):

    start = true
    self.app.appType = 'enterprise'
    if not self.app.connectionList.has_key('enterprise'):
      dlg = wxDlgMakeConnection(self,-1,'Setup Enterprise connection',self.app,
                                'enterprise')
      if dlg.ShowModal() != wxID_OK:
        start = false
      dlg.Destroy()  
    if start:
      self.windowEnterprise = wxDlgMain(self.splitter, -1, self.app)
      self.windowEnterprise.initPanel(self.app)
      window = self.splitter.GetWindow2()
      self.splitter.ReplaceWindow(window, self.windowEnterprise)
      window.Destroy()
      self.SetStatusText('Enterprises application started ....',0)

  # onButtonProjects
  def onButtonProjects(self,event):

    start = true
    self.app.appType = 'projects'
    if not self.app.connectionList.has_key('project'):
      dlg = wxDlgMakeConnection(self,-1,'Setup Project connection',self.app,'project')
      if dlg.ShowModal() != wxID_OK:
        start = false
      dlg.Destroy()
    if start:  
      self.windowProjects = wxPanProjectBrowser(self.splitter, -1, self.app)
      window = self.splitter.GetWindow2()
      self.splitter.ReplaceWindow(window, self.windowProjects)
      window.Destroy()
      self.SetStatusText('Projects application started ....',0)
