#!/usr/bin/env python
#----------------------------------------------------------------------------
# Name:         wxPanProjectBrowser.py
# Author:       Bjoern
# Created:      09/26/2001
# Copyright:    (c) 2001 SKYRiX Software AG
#----------------------------------------------------------------------------

from wxPython.wx import *
from wxSkyClient_wdr import *
from wxDlgEditFile import wxDlgEditFile
from wxDlgEditAttributes import wxDlgEditAttributes

class wxPanProjectBrowser(wxPanel):

  # __init__
  def __init__(self, parent, id, app):
    wxPanel.__init__(self, parent, -1, size = (700, 500),
                      style=wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE)

    # get the controls
    dlgProjectBrowserMain(self, true)

    self.splitter = wxPyTypeCast(self.FindWindowById(ID_SPLITTER),'wxSplitterWindow')
   
    self.tree = wxTreeCtrl(self.splitter, -1)
    self.list = wxListCtrl(self.splitter, -1,size=(400,400),
                           style=wxLC_REPORT|wxLC_SINGLE_SEL)

    self.splitter.SetMinimumPaneSize(120)
    self.splitter.SplitVertically(self.tree,self.list)
    self.splitter.SetSashPosition(120)

    self.fileList = []
    self.app = app
    self.fm = app.connectionList['project']

    self.keys = self.app.config.getValue('project.keys')
    self.currentPath = '/'

    EVT_LIST_ITEM_ACTIVATED(self.list,-1,self.onItemActivated)
    EVT_COMMAND_RIGHT_CLICK(self.list,-1,self.onRightClick)
    EVT_RIGHT_UP(self.list,self.onRightClick)
    EVT_LIST_BEGIN_LABEL_EDIT(self.list,-1,self.onBeginLabelEdit)
    EVT_LIST_END_LABEL_EDIT(self.list,-1,self.onEndLabelEdit)
    EVT_TREE_ITEM_ACTIVATED(self.tree, -1, self.onTreeItemActivated)

    # set columns and columnwidth
    counter = 0
    for key in self.keys:
      self.list.InsertColumn(counter, key[0]);
      counter = counter + 1

    self.list.SetColumnWidth(0, 160)
    self.list.SetColumnWidth(1, 100)
    self.list.SetColumnWidth(2, 300)

    self.onRoot(-1)
  
  # onItemActivated
  # -> reload clicked directory and sync tree
  def onItemActivated(self,event):
    dirName = self.list.GetItemText(event.m_itemIndex)

    if not '.' in dirName:
      if self.currentPath == "/":
        self.currentPath = self.currentPath + dirName
      else:
        self.currentPath = self.currentPath + '/' + dirName
      self.reloadDirectory(self.currentPath)
      self.getItemForPathAndRedraw(self.currentPath)
    else:
      for attrs in self.fileList:
        if attrs['NSFileName'] == dirName: break
      if attrs['NSFileType'] == 'NSFileTypeRegular':
        if attrs['NSFileMimeType'] == 'text/plain':
          dlg = wxDlgEditFile(self,-1,'Edit file '+dirName, self.app, 
                              self.currentPath + '/'+dirName)

  # onRescan
  # -> rescan current directory
  def onRescan(self,event):
    self.reloadDirectory()

  # onRoot
  # -> reload the / directory in the browser
  def onRoot(self, event):
    if self.app.checkIfConnected('project'):
      self.root = self.tree.AddRoot("/")
      self.addTreeToItem(self.root,"/")
      self.Show(true)

  # onRightClick
  # show the popup menu
  def onRightClick(self,event):
    menu = wxMenu()
    
    menu.Append(0,"Up")
    EVT_MENU(self, 0, self.onDirUp)
    menu.Append(7,"New Directory")
    EVT_MENU(self, 7, self.onNewDirectory)
    menu.Append(4,"New File")
    EVT_MENU(self, 4, self.onNew)
    
    menu.AppendSeparator()
    menu.Append(1,"Delete")
    EVT_MENU(self, 1, self.onDelete)
    menu.Append(2,"Edit")
    EVT_MENU(self, 2, self.onEdit)
    menu.Append(3,"Rename")
    EVT_MENU(self, 3, self.onRename)
    menu.Append(5,"Copy")
    EVT_MENU(self,5, self.onCopy)
    menu.Append(6,"Move")
    EVT_MENU(self,6, self.onMove)
    menu.Append(8,"Attributes")
    EVT_MENU(self,8, self.onAttributes)
   
    self.list.PopupMenu(menu, wxPoint(event.GetX(),event.GetY()))
    menu.Destroy()
    event.Skip()

  # onBack
  # -> get back one step/one dir up
  def onBack(self,event):
    if self.app.checkIfConnected('project'):
      self.getBackOneStep()

  # getItemForPathAndRedraw
  # -> find item for current path and add a subitems
  def getItemForPathAndRedraw(self,path,add=true):
    splitPath = string.split(path,'/')
    item = (self.root , 0)
    if path == '/': self.addTreeToItem(self.root,"/")

    for p in range(len(splitPath)):
      if p>0:
        subitem = self.tree.GetFirstChild(item[0],0)
        for i in range(self.tree.GetChildrenCount(item[0])):
          if self.tree.GetItemText(subitem[0]) == splitPath[p]:
            item = subitem
            break
          else:
            subitem = self.tree.GetNextChild(item[0],subitem[1])
    if add: self.addTreeToItem(subitem[0],path)
    else: 
      self.tree.Delete(subitem[0])
      self.reloadDirectory()

  # onTreeItemActivated
  # -> get the path for the item and add subitems
  def onTreeItemActivated(self,event):
    targetPath = ''
    theItem = event.GetItem()
    while theItem != self.root:
      targetPath = self.tree.GetItemText(theItem) + '/' + targetPath
      theItem = self.tree.GetItemParent(theItem)
    if event.GetItem() != self.root:
      self.addTreeToItem(event.GetItem(),"/"+targetPath[:len(targetPath)-1])
    else:
      self.addTreeToItem(event.GetItem(),"/")

  # addTreeToItem
  # -> add subtree with subdirs to current item
  def addTreeToItem(self,item,directory):
    if self.app.checkIfConnected('project'):
      # parse the / dir of the project and add all
      # directories to the tree
      counter = 0
      ds = self.fm.dataSourceAtPath(directory)
      self.fileList = ds.fetchObjects()
      for attrs in self.fileList:
        pfile = attrs['NSFilePath']
        isdir = attrs['NSFileType'] == 'NSFileTypeDirectory'
        # check if directory is already added (parse through the tree
        # and compare the labels
        if isdir:
          theSibling = self.tree.GetFirstChild(item,0)
          theString = pfile[string.rfind(pfile,"/")+1:]
          stringFound = false
          while self.tree.GetItemText(theSibling[0]) != "":
            if self.tree.GetItemText(theSibling[0]) == theString:
              stringFound = true
            theSibling = self.tree.GetNextChild(item,theSibling[1])
          if not stringFound:
            subitem = self.tree.AppendItem(item,pfile[string.rfind(pfile,"/")+1:])

    self.reloadDirectory(directory)
    self.tree.Expand(item)
    self.tree.SelectItem(item)

  # onBeginLabelEdit
  def onBeginLabelEdit(self,event):
    "rename the file/directory by editing the label"
    theName = self.list.GetItemText(self.GetNextItem(-1,wxLIST_NEXT_ALL,
                               wxLIST_STATE_SELECTED))
    slash = '/'
    if self.currentPath == '/': slash = ''
    self.sourceItem = self.currentPath + slash + theName

  # onEndLabelEdit
  def onEndLabelEdit(self,event):
    "rename the file/directory by editing the label"
    slash = '/'
    if self.currentPath == '/': slash = ''
    targetItem = self.currentPath + slash + event.GetLabel()
    if not self.fm.fileExistsAtPath(targetItem):
      self.fm.movePathToPath(self.sourceItem,targetItem)
    else: 
      event.Veto()
      self.app.frame.SetStatusText("Target exists")
 
    self.getItemForPathAndRedraw(self.sourceItem, false)
    theString = targetItem[:string.rfind(targetItem,"/")]
    if theString == '': theString = "/"
    self.getItemForPathAndRedraw(theString)


  # onAttributes
  # -> get attributes for current file/directory
  def onAttributes(self,event):
    theName = self.list.GetItemText(self.list.GetNextItem(-1,wxLIST_NEXT_ALL,
			       wxLIST_STATE_SELECTED))
    slash = '/'
    if self.currentPath == '/': slash = ''
    directory = self.currentPath + slash + theName
    dlg = wxDlgEditAttributes(self, -1, directory, self.app)

  # onNewDirectory
  # -> create a new directory
  def onNewDirectory(self,event):
    dirName = '/subdir'
    if self.currentPath == '/': dirName = 'subdir'
    dlg = wxTextEntryDialog(self,'New Path','New Directory',self.currentPath + dirName)
    if dlg.ShowModal() == wxID_OK:
      self.fm.createDirectoryAtPath(dlg.GetValue())
      self.reloadDirectory()
      self.app.getItemForPathAndRedraw(self.currentPath)    
 
  # onCopy
  # -> copy a file/directory to another location within the tree
  def onCopy(self,event):
    theName = self.list.GetItemText(self.list.GetNextItem(-1,wxLIST_NEXT_ALL,
			       wxLIST_STATE_SELECTED))
    slash = '/'
    if self.currentPath == '/': slash = ''
    dlg = wxTextEntryDialog(self,'target :','Copy File',self.currentPath+slash + theName)
    if dlg.ShowModal() == wxID_OK:
      for attrs in self.fileList:
        if attrs['NSFileName'] == theName: break
      
      self.fm.copyPathToPath(self.currentPath + slash + theName, dlg.GetValue())
      if attrs['NSFileType'] == 'NSFileTypeDirectory':
        theString = dlg.GetValue()[:string.rfind(dlg.GetValue(),"/")]
        self.getItemForPathAndRedraw(theString)
      self.reloadDirectory()

  # onMove
  # -> move a file/(directory) to another location within the tree
  def onMove(self,event):
    print self.currentPath
    theName = self.list.GetItemText(self.list.GetNextItem(-1,wxLIST_NEXT_ALL,
			       wxLIST_STATE_SELECTED))
    slash = '/'
    if self.currentPath == '/': slash = ''
    dlg = wxTextEntryDialog(self,'target :','Move File',self.currentPath+slash + theName)
    if dlg.ShowModal() == wxID_OK:
      for attrs in self.fileList:
        if attrs['NSFileName'] == theName: break

      # doesn't work for directories yet (dunno why)
      self.fm.movePathToPath(self.currentPath + slash + theName, dlg.GetValue())
      if attrs['NSFileType'] == 'NSFileTypeDirectory':
        self.app.frame.getItemForPathAndRedraw(self.currentPath + slash + theName, false)
        theString = dlg.GetValue()[:string.rfind(dlg.GetValue(),"/")]
        self.app.getItemForPathAndRedraw(theString)
      self.reloadDirectory()

  # onDirUp
  # -> get one level up in the directory tree
  def onDirUp(self,event):
    self.getBackOneStep()

  # onDelete
  # -> delete a file/directory
  def onDelete(self,event):
    theName = self.list.GetItemText(self.list.GetNextItem(-1,wxLIST_NEXT_ALL,
                               wxLIST_STATE_SELECTED))
    slash = '/'
    if self.currentPath == '/': slash = ''
    thePath = self.currentPath + slash + theName
    for attrs in self.fileList:
      if attrs['NSFileName'] == theName: break
    dlg=wxMessageDialog(self, 'Really ?', 'Question', wxYES_NO)
    if dlg.ShowModal() == wxID_YES:
      self.fm.removeFileAtPath(thePath)
      if attrs['NSFileType'] == 'NSFileTypeDirectory':
        self.app.getItemForPathAndRedraw(thePath,false) 
      self.reloadDirectory()
    dlg.Destroy()

  # onEdit
  # -> edit a file
  def onEdit(self,event):
    theName = self.list.GetItemText(self.list.GetNextItem(-1,wxLIST_NEXT_ALL,
                               wxLIST_STATE_SELECTED))
    slash = '/'
    if self.currentPath == '/': slash = ''
    thePath = self.currentPath + slash + theName
    for attrs in self.fileList:
      if attrs['NSFileName'] == theName: break
    if attrs['NSFileType'] == 'NSFileTypeRegular':
      if attrs['NSFileMimeType'] == 'text/plain':
        dlg = wxDlgEditFile(self,-1,'Edit file '+thePath, self.app, thePath)
    
  # onRename
  # -> rename a file/directory
  def onRename(self,event):
    item = self.list.GetNextItem(-1,wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)
    self.list.EditLabel(item)

  # onNew
  # -> add a new file
  def onNew(self,event):
    fileName = '/new.txt'
    if self.currentPath == '/': fileName = 'new.txt'
    thedlg = wxTextEntryDialog(self,'name :','New File',self.currentPath + fileName)
    if thedlg.ShowModal() == wxID_OK:
      dlg = wxDlgEditFile(self,thedlg.GetValue())
      dlg.ShowModal()
      dlg.Destroy()
  
  # getBackOneStep
  # -> jump to last dir/one level up
  def getBackOneStep(self):
    self.currentPath = self.currentPath[:string.rfind(self.currentPath,'/')]

    if len(self.currentPath) == 0: 
      self.currentPath = "/"
    self.reloadDirectory(self.currentPath)
    self.tree.SelectItem(self.tree.GetItemParent(self.tree.GetSelection()))
    self.getItemForPathAndRedraw(self.currentPath)

  # reloadDirectory
  # -> reload the table's current directory
  def reloadDirectory(self,directory=''):

    self.filestructs = {};
    if directory != '':
      self.currentPath = directory
    else:
      directory = self.currentPath

    if self.app.checkIfConnected('project'):
      for attrs in self.fileList:
        tu = []
        for key in self.keys:
          if (key[0] == "NSFileSize") & (attrs['NSFileType'] == 'NSFileTypeDirectory'):
            tu.append("(dir)")
          else:
            tu.append(attrs[key[0]])
        self.filestructs[attrs['NSFilePath']] = tuple(tu)
    self.redrawListCtrl()

  # redrawListCtrl
  # -> clear the list and redraw all items
  def redrawListCtrl(self):
    self.list.DeleteAllItems()

    items = self.filestructs.items()

    for counter in range(len(items)):
      key, data = items[counter]
      self.list.InsertStringItem(counter,data[0])
      self.list.SetStringItem(counter,1, "%s" % data[1])
      self.list.SetStringItem(counter,2, data[2])
